@@ -200,19 +200,26 @@ def sample(
     model: nn.Module,
     *,
     seq_len,
-    api_start_token_id,
-    select_api_start_id_top_k = 10,
     prime: Optional[torch.Tensor] = None,
     positions: Optional[torch.Tensor] = None,
     batch_size = 1,
     eos_token_id = None,
     sos_token_id = 1,
     temperature = 0.,
-    pad_id = 0
+    pad_id = 0,
+    call_api_only_once = False,
+    api_start_token_id = None,
+    auto_select_api_start_token_when_topk = False,
+    select_api_start_id_top_k = 10,
 ):
     device = next(model.parameters()).device
     max_seq_len = seq_len + 1
 
+    # validate
+
+    if call_api_only_once:
+        assert exists(api_start_token_id)
+
     # prime
 
     if exists(prime):
@@ -257,17 +264,18 @@ def sample(
         logits = model(output)
         last_logits = logits[batch_indices, position_indices]
 
-        if not exists(api_token_mask):
-            num_tokens = last_logits.shape[-1]
-            api_token_mask = create_api_token_mask(num_tokens, api_start_token_id)
-            api_token_mask = api_token_mask.to(device)
-
-        api_called = (output == api_start_token_id).any(dim = -1)
-
         # this will ensure that each batch token sequence will have at most one <api> token
 
-        logit_mask = api_token_mask & rearrange(api_called, 'b -> b 1 1')
-        last_logits = last_logits.masked_fill(logit_mask, -torch.finfo(last_logits.dtype).max)
+        if call_api_only_once:
+            if not exists(api_token_mask):
+                num_tokens = last_logits.shape[-1]
+                api_token_mask = create_api_token_mask(num_tokens, api_start_token_id)
+                api_token_mask = api_token_mask.to(device)
+
+            api_called = (output == api_start_token_id).any(dim = -1)
+
+            logit_mask = api_token_mask & rearrange(api_called, 'b -> b 1 1')
+            last_logits = last_logits.masked_fill(logit_mask, -torch.finfo(last_logits.dtype).max)
 
         # greedy sample (but could be made non-greedy)
 
@@ -278,11 +286,12 @@ def sample(
         # seems to be an important hack in the paper
         # it seems like this paper will take a lot more follow up research to be viable
 
-        top_token_ids = last_logits.topk(select_api_start_id_top_k, dim = -1).indices
-        has_api_token_in_topk = (top_token_ids == api_start_token_id).any(dim = -1)
-        should_auto_select_api_token = has_api_token_in_topk & ~rearrange(api_called, 'b -> b 1')
+        if auto_select_api_start_token_when_topk:
+            top_token_ids = last_logits.topk(select_api_start_id_top_k, dim = -1).indices
+            has_api_token_in_topk = (top_token_ids == api_start_token_id).any(dim = -1)
+            should_auto_select_api_token = has_api_token_in_topk & ~rearrange(api_called, 'b -> b 1')
 
-        sampled = sampled.masked_fill(should_auto_select_api_token, api_start_token_id)
+            sampled = sampled.masked_fill(should_auto_select_api_token, api_start_token_id)
 
         # set the sampled tokens at the right curosr positions
 
